local SCA, Ace3 = unpack(select(2,...))
Ace3.version = GetAddOnMetadata(..., 'Version')

local function InitCritOptions(options)
	local db = Ace3.db.char

	if (db.audio.crits.damage.custom.customType == "file") then
		options.args.damageTab.args.CustomSound.args.audioLocation.width = "full"
	else
		options.args.damageTab.args.CustomSound.args.audioLocation.width = "double"
	end
	
	if (db.audio.crits.heals.custom.customType == "file") then
		options.args.healingTab.args.CustomSound.args.audioLocation.width = "full"
	else
		options.args.healingTab.args.CustomSound.args.audioLocation.width = "double"
	end
end

local function VerifyCustomPath(value,db,trigger,inputType)
	if (strfind(value,".mp3")) then
		value = gsub(value,".mp3",'')
	elseif (strfind(value,".ogg")) then
		value = gsub(value,".ogg",'')
	end
	
	if (strsub(value,#value) ~= [[\]] and db.audio[trigger][inputType].custom.customType == "pack" and #value > 0) then
		value = value..[[\]]
	elseif (strsub(value,#value) == [[\]] and db.audio[trigger][inputType].custom.customType == "file") then
		value = strsub(value,0,#value - 1)
	end
	
	return value
end
--/script local a = { [51505] = "Lava Burst",[188196] = "Lightning Bolt",[188443] = "Chain Lightning"}; for v,i in pairs(a) do print(tostring(v)..", "..tostring(i)) end
--/script local a = { ["Lava Burst"] = true, ["Chain Lightning"] = true,} for v,i in pairs(a) do print("hello") end
local function ReloadCritIgnoreList(options)
	local db = Ace3.db.char
	local args = {}
	local ctr = 1
	
	--table.sort(db.ignore)
	for k,v in pairs(db.ignore) do
		args["spell"..ctr] = {
			order = ctr,
			type = "toggle",
			desc = "Spell ID: "..k,
			descStyle = "inline",
			name = "|cFFEBC000"..v.."|r",
			get = function() 
				return true
			end,
			set = function(self,value)
				db.ignore[k] = nil
				ReloadCritIgnoreList(options)
			end,
		}
		ctr = ctr + 1
	end
	
	options.args.ignoreTab.args.ignoreList.args = args
end


local crit_options

local function GetCritOptions()
	if (not crit_options) then
		--local db = Ace3.db.char
		local db = Ace3.db.char
		
		crit_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "Crit Sound Options",
			desc = '',
			args = {
				damageTab = {
					name = COMBATLOG_HIGHLIGHT_DAMAGE,
					order = 0,
					type = "group",
					args = {
						sOptions = {
							name = "Sound Options",
							order = 0,
							type = "group",
							guiInline = true,
							args = {
								packs = {
									order = 1,
									type = "toggle",
									name = "Sound Packs",
									get = function() 
										return db.audio.crits.damage.options.source.packs
									end,
									set = function(self,value)
										db.audio.crits.damage.options.source.packs = value
										
										if (value) then
											db.audio.crits.damage.options.source.custom = false
										end
									end,
								},
								custom = {
									order = 2,
									type = "toggle",
									name = "Custom Audio",
									get = function() 
										return db.audio.crits.damage.options.source.custom
									end,
									set = function(self,value)
										db.audio.crits.damage.options.source.custom = value
										
										if (value) then
											db.audio.crits.damage.options.source.packs = false
										end
									end,
								},
								audioSource = {
									order = 3,
									type = "select",
									name = "Sound Channel",
									desc = "Choose which channel for the audio to play",
									get = function()
										return db.audio.crits.damage.options.channel
									end,
									set = function(self,value)
										db.audio.crits.damage.options.channel = value
									end,
									values = {
										["Master"] = MASTER,
										["SFX"] = SOUND,
										["Music"] = MUSIC_VOLUME,
										["Ambience"] = AMBIENCE_VOLUME,
										["Dialog"] = DIALOG_VOLUME,
									},
								},
								DmgGroup = {
									name = "Damage",
									order = 4,
									type = "group",
									guiInline = true,
									args = {
										Damage = {
											order = 1,
											type = "toggle",
											name = "Damage Spells",
											get = function() 
												return db.audio.crits.damage.options.spells
											end,
											set = function(self,value)
												db.audio.crits.damage.options.spells = value
											end,
										},
										DoTs = {
											order = 2,
											type = "toggle",
											name = "DoT Spells",
											get = function() 
												return db.audio.crits.damage.options.dots
											end,
											set = function(self,value)
												db.audio.crits.damage.options.dots = value
											end,
										},
										PetDamage = {
											order = 3,
											type = "toggle",
											name = "Pet Damage Spells",
											get = function() 
												return db.audio.crits.damage.options.pet
											end,
											set = function(self,value)
												db.audio.crits.damage.options.pet = value
											end,
										},
										Threshold = {
											order = 4,
											type = "range",
											name = "Damage Trigger Threshold",
											desc = "Restrict the audio queues to a specific damage threshold",
											min = 0,
											max = 10000000,
											step = 25000,
											bigStep = 25000,
											get = function() 
												return db.audio.crits.damage.options.threshold
											end,
											set = function(self,value)
												db.audio.crits.damage.options.threshold = value
											end,
											width = "full",
										},
									},
								},
							},
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 1,
							type = "group",
							guiInline = true,
							disabled = function()
								if (db.audio.crits.damage.options.source.custom) then
									return true
								else
									return false
								end
							end,
							args = {
								packSelect = {
									order = 3,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									get = function()
										return db.audio.crits.damage.packs.index
									end,
									set = function(self,value)										
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.crits.damage.packs.index = value
											db.audio.crits.damage.packs.text = audio.text
											db.audio.crits.damage.packs.value = audio.value
											db.audio.crits.damage.packs.path = audio.path
											db.audio.crits.damage.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 2,
							type = "group",
							guiInline = true,
							disabled = function()
								if (db.audio.crits.damage.options.source.packs) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.crits.damage.custom.customType
									end,
									set = function(self,value)
										db.audio.crits.damage.custom.customType = value
										
										if (value == "file") then
											crit_options.args.damageTab.args.CustomSound.args.audioLocation.width = "full"
										else
											crit_options.args.damageTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.crits.damage.custom.fileType
									end,
									set = function(self,value)
										db.audio.crits.damage.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.crits.damage.custom.path,db,'crits','damage')
									end,
									set = function(self,value)								
										db.audio.crits.damage.custom.path = VerifyCustomPath(value,db,'crits','damage')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.crits.damage.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.crits.damage.custom.max
									end,
									set = function(self,value)
										db.audio.crits.damage.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.crits.damage.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.crits.damage.custom.path..tostring(math.random(1,db.audio.crits.damage.custom.max))..db.audio.crits.damage.custom.fileType,db.audio.crits.damage.options.channel)) then
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.crits.damage.custom.path..db.audio.crits.damage.custom.fileType,db.audio.crits.damage.options.channel)) then
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				healingTab = {
					name = SHOW_COMBAT_HEALING,
					order = 1,
					type = "group",
					args = {
						sOptions = {
							name = "Sound Options",
							order = 0,
							type = "group",
							guiInline = true,
							args = {
								packs = {
									order = 1,
									type = "toggle",
									name = "Sound Packs",
									get = function() 
										return db.audio.crits.heals.options.source.packs
									end,
									set = function(self,value)
										db.audio.crits.heals.options.source.packs = value
										
										if (value) then
											db.audio.crits.heals.options.source.custom = false
										end
									end,
								},
								custom = {
									order = 2,
									type = "toggle",
									name = "Custom Audio",
									get = function() 
										return db.audio.crits.heals.options.source.custom
									end,
									set = function(self,value)
										db.audio.crits.heals.options.source.custom = value
										
										if (value) then
											db.audio.crits.heals.options.source.packs = false
										end
									end,
								},
								audioSource = {
									order = 3,
									type = "select",
									name = "Sound Channel",
									desc = "Choose which channel for the audio to play",
									get = function()
										return db.audio.crits.heals.options.channel
									end,
									set = function(self,value)
										db.audio.crits.heals.options.channel = value
									end,
									values = {
										["Master"] = MASTER,
										["SFX"] = SOUND,
										["Music"] = MUSIC_VOLUME,
										["Ambience"] = AMBIENCE_VOLUME,
										["Dialog"] = DIALOG_VOLUME,
									},
								},
								HealGroup = {
									name = "Healing",
									order = 5,
									type = "group",
									guiInline = true,
									args = {
										Healing = {
											order = 1,
											type = "toggle",
											name = "Healing Spells",
											get = function() 
												return db.audio.crits.heals.options.spells
											end,
											set = function(self,value)
												db.audio.crits.heals.options.spells = value
											end,
										},
										HoTs = {
											order = 2,
											type = "toggle",
											name = "Hot Spells",
											get = function() 
												return db.audio.crits.heals.options.hots
											end,
											set = function(self,value)
												db.audio.crits.heals.options.hots = value
											end,
										},
										PetHealing = {
											order = 3,
											type = "toggle",
											name = "Pet Healing Spells",
											get = function() 
												return db.audio.crits.heals.options.pet
											end,
											set = function(self,value)
												db.audio.crits.heals.options.pet = value
											end,
										},
										Threshold = {
											order = 4,
											type = "range",
											name = "Healing Trigger Threshold",
											desc = "Restrict the audio queues to a specific healing threshold",
											min = 0,
											max = 10000000,
											step = 25000,
											bigStep = 25000,
											get = function() 
												return db.audio.crits.heals.options.threshold
											end,
											set = function(self,value)
												db.audio.crits.heals.options.threshold = value
											end,
											width = "full",
										},
									},
								},
							},
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 1,
							type = "group",
							guiInline = true,
							disabled = function()
								if (db.audio.crits.heals.options.source.custom) then
									return true
								else
									return false
								end
							end,
							args = {
								packSelect = {
									order = 3,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									get = function()
										return db.audio.crits.heals.packs.index
									end,
									set = function(self,value)
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.crits.heals.packs.index = value
											db.audio.crits.heals.packs.text = audio.text
											db.audio.crits.heals.packs.value = audio.value
											db.audio.crits.heals.packs.path = audio.path
											db.audio.crits.heals.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 2,
							type = "group",
							guiInline = true,
							disabled = function()
								if (db.audio.crits.heals.options.source.packs) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.crits.heals.custom.customType
									end,
									set = function(self,value)
										db.audio.crits.heals.custom.customType = value
										
										if (value == "file") then
											crit_options.args.healingTab.args.CustomSound.args.audioLocation.width = "full"
										else
											crit_options.args.healingTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.crits.heals.custom.fileType
									end,
									set = function(self,value)
										db.audio.crits.heals.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.crits.heals.custom.path,db,'crits','heals')
									end,
									set = function(self,value)								
										db.audio.crits.heals.custom.path = VerifyCustomPath(value,db,'crits','heals')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.crits.heals.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.crits.heals.custom.max
									end,
									set = function(self,value)
										db.audio.crits.heals.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.crits.heals.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.crits.heals.custom.path..tostring(math.random(1,db.audio.crits.heals.custom.max))..db.audio.crits.heals.custom.fileType,db.audio.crits.heals.options.channel)) then
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.crits.heals.custom.path..db.audio.crits.heals.custom.fileType,db.audio.crits.heals.options.channel)) then
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				ignoreTab = {
					name = IGNORE,
					order = 2,
					type = "group",
					args = {
						input = {
							name = "Spell Input",
							order = 0,
							type = "group",
							desc = "Click on a checkbox to unignore a spell",
							guiInline = true,
							args = {
								spellInfo = {
									type = "input",
									order = 1,				
									name = "Spell ID / Name",
									desc = "Input spell id, or name, to be ignored by the addon.\n\n|cFFFF0000NOTE|r: Spell IDs are more reliable. Downloading the addon \"|cFF00FF00idTip|r\" shows Spell IDs on tooltips.\n\nIf entering a spell's name results in an error, you will have to use the spell's ID",
									get = function()
											return SCA.temp.ignore
									end,
									set = function(self,value)
										local name,_,_,_,_,_,spellID = GetSpellInfo(value)
										
										if (name and not db.ignore[spellID] and IsUsableSpell(name)) then
											crit_options.args.ignoreTab.args.input.args.submit.disabled = false
											crit_options.args.ignoreTab.args.input.args.errorMsg.name = " "
											SCA.temp.ignore = value
										else
											crit_options.args.ignoreTab.args.input.args.submit.disabled = true
											if (db.ignore[spellID]) then
												crit_options.args.ignoreTab.args.input.args.errorMsg.name = "|cFFFF0000Spell is already ignored|r"
											elseif (name and not IsUsableSpell(name)) then
												crit_options.args.ignoreTab.args.input.args.errorMsg.name = "|cFFFF0000You cannot use that spell|r"
											else
												crit_options.args.ignoreTab.args.input.args.errorMsg.name = "|cFFFF0000Spell not found|r"
											end
											
											SCA.temp.ignore = ''
										end
										
									end,
								},
								submit = {
									order = 2,
									type = "execute",
									name = "Ignore",
									disabled = true,
									func = function()
										local name,_,_,_,_,_,spellID = GetSpellInfo(SCA.temp.ignore)
										
										--if (name) then
											db.ignore[spellID] = name
											crit_options.args.ignoreTab.args.input.args.submit.disabled = true
											SCA.temp.ignore = ''
											--ignore_options.args.input.args.errorMsg.name = ""
										--else
											--ignore_options.args.input.args.errorMsg.name = "|cFFFF0000Spell not found|r"
										--end
										ReloadCritIgnoreList(GetCritOptions())
									end,
								},
								errorMsg = {
									order = 3,
									type = 'description',
									name = " ",
								},
							},
						},
						ignoreList = {
							name = "Ignored Spells",
							order = 1,
							type = "group",
							dialogInline = true,
							args = {},
							--[[	fileType = {
									order = 1,
									type = "select",
									name = "",
									desc = "Choose the file type for your audio files",
									guiInline = true,
									get = function()
										return db.audio.custom.fileType
									end,
									set = function(self,value)
										db.audio.custom.fileType = value
									end,
									values = function()
										local spells = {}
										for k,_ in pairs(db.ignore) do
											local name = GetSpellInfo(k)
											
											spells[k] = name
										end
										return spells
									end,
									width = "full"
								},
							},]]
						},
					},
				},
			},
		}
	end
	
	return crit_options
end

local ignore_options

local function GetIgnoreOptions()
	if (not ignore_options) then
		local db = Ace3.db.char
		
		ignore_options = {
			type = "group",
			order = 1,
			name = "Ignore Options",
			desc = '',
			args = {
				
			},
		}
		
	end
	
	return ignore_options
end

function Ace3:SetupOptions()
	local ACFG = LibStub("AceConfig-3.0")
	ACFG:RegisterOptionsTable("CustomAudio Crit Sounds", GetCritOptions)

	local ACD = LibStub("AceConfigDialog-3.0")
	ACD:AddToBlizOptions("CustomAudio Crit Sounds", "Crit Sounds", "CustomAudio")
end

local function UpdateInterfaceSettings()
	local options = GetCritOptions()
	InitCritOptions(options)
	ReloadCritIgnoreList(options)
end

InterfaceOptionsFrame:HookScript("OnShow",function(self)
	UpdateInterfaceSettings()
end)